#include "sjog.h"
#include "sjog_config.h"
#include "sjog_list.h"
#include "sjog_sonypi.h"

#include <libintl.h>
#define _(String) gettext (String)

/* -------------------------------------------------------------------------- */

sjog_button_cmd_t *sjog_button_cmd_new();
sjog_button_cmd_t *sjog_parse_button(gchar *line, gboolean *err);

/**
 * Should probably be somewhere else - rearrange at will
 */
char *buttonnames[] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    "fn-esc",
    "fn-f1",
    "fn-f2",
    "fn-f3",
    "fn-f4",
    "fn-f5",
    "fn-f6",
    "fn-f7",
    "fn-f8",
    "fn-f9",
    "fn-f10",
    "fn-f11",
    "fn-f12",
    "fn-1",
    "fn-2",
    "fn-D",
    "fn-E",
    "fn-F",
    "fn-S",
    "fn-B",
    "bluetooth",
    "pkey-p1",
    "pkey-p2",
    "pkey-p3",
    NULL,
    "lid-closed",
    "lid-opened",
    "bluetooth-on",
    "bluetooth-off",
};


/*
 * Read the sjog config file and put the parsed entries in a list.
 */
gint
sjog_init_list_entries()
{

   FILE *config_stream;
   gchar line[LINE_MAX];
   gboolean err = FALSE;
   gchar rcname[LINE_MAX];
   gchar section[LINE_MAX];

   strcpy(section, "jogdial");

   /* 1st try to get the user config file */
   snprintf(rcname, LINE_MAX, "%s/.sjogrc", getenv("HOME"));

   config_stream = fopen(rcname, "r");

   /* if the user has no config file we try to get the system one */
   if (config_stream == NULL)
   {
      config_stream = fopen(strdup(SYSCONFDIR "/sjogrc"), "r");
      if (config_stream == NULL)
      {
         fprintf(stderr, _("Can't open config file !\n"));
         exit(1);
      }
   }

   /* foreach line, parse it and fill a list of sjog_cmd_t */
   while (!err && fgets(line, sizeof(line), config_stream) != NULL)
   {
      if (line[0]=='[') {
         gchar *tmp;
         for (tmp=line+1;*tmp!='\0';tmp++) {
            if (*tmp==']') {
               *tmp=0;
               strncpy(section, line+1, sizeof(section));
            }
         }
      } else {

         if (section!=NULL && strcmp(section, "buttons")==0) {
            sjog_button_cmd_t *cmd;

            cmd = sjog_parse_button(line, &err);

            if (err == TRUE)
            {
               fclose(config_stream);
               fprintf(stderr, "Error in config file !\n");
               exit(1);
            }
            else if (cmd != NULL && cmd->path != NULL && cmd->command!=0)
            {
               D("sjog_init_config: button=%d path=%s\n", cmd->command, cmd->path);
               buttons = g_list_append(buttons, cmd);
            }
            
         } else {
      sjog_cmd_t *cmd;

      cmd = sjog_parse(line, &err);
      if (err == TRUE)
      {
         fclose(config_stream);
         fprintf(stderr, _("Error in config file !\n"));
         exit(1);
      }
      else if (cmd == NULL || cmd->path == NULL)
      {
         /* white or comment line, skip it */
         continue;
      }

      D("sjog_init_config: name=%s path=%s\n", cmd->name, cmd->path);

      list = g_list_append(list, cmd);
         }
      }
   }
      D("list contains %d items\n", g_list_length(list));
}

/* -------------------------------------------------------------------------- */

sjog_button_cmd_t *sjog_parse_button(gchar *line, gboolean *err)
{
   gchar *tmp, *path;
   int i;

   for (tmp = line; *tmp != '\0'; tmp++)
   {
      if (*tmp == '#' || *tmp == '\n')
      {
         *tmp = '\0';
         break;
      }
   }
   while (tmp != line)
   {
      tmp--;
      if (*tmp == ' ' || *tmp == '\t')
      {
         *tmp = '\0';
      }
      else
      {
         break;
      }
   }
   while (*line == ' ' || *line == '\t')
   {
      line++;
   }
   if (*line == '\0')                        /* premature end of line */
   {
      return NULL;
   }

   for (path=line;*path!=' ' && *path!='\t' && *path!='\0';path++);

   while (*path == ' ' || *path == '\t') {
      *path++='\0';
   }
   if (*path=='\0') {
       *err = TRUE;
       return NULL;
   }
   for (i=0;i<=SONYPI_MAX_EVENT;i++) {
       if (buttonnames[i]!=NULL && strcasecmp(buttonnames[i], line)==0) {
           sjog_button_cmd_t *cmd = sjog_button_cmd_new();
           cmd->command = i;
           cmd->path = strdup(path);
           return cmd;
       }
   }
   return NULL;
}

/*
 * takes a line and parse a name/path pair in it.
 */
sjog_cmd_t *
sjog_parse(gchar * line, gboolean * err)
{
   gchar *tmp, *name;
   sjog_cmd_t *sjog_cmd;

   *err = FALSE;

   sjog_cmd = sjog_cmd_new();

   if (! sjog_cmd)
      return NULL;

   /* strip comment or final \n */

   for (tmp = line; *tmp != '\0'; tmp++)
   {
      if (*tmp == '#' || *tmp == '\n')
      {
         *tmp = '\0';
         break;
      }
   }

   /* strip trailing whitespaces */

   while (tmp != line)
   {
      tmp--;
      if (*tmp == ' ' || *tmp == '\t')
      {
         *tmp = '\0';
      }
      else
      {
         break;
      }
   }

   /* find the name */

   tmp = line;
   while (*tmp == ' ' || *tmp == '\t')
   {
      tmp++;
   }
   if (*tmp == '\0')                        /* premature end of line */
   {
      return NULL;
   }

   /* spaces are allowed if the name is put between doublequotes */
   if (*tmp == '"')
   {
      /* skip first " */
      tmp++;

      /* name will start from here */
      name = tmp;

      /* allow everything except doublequotes and EOL */
      while (*tmp != '\0' && *tmp != '"')
      {
         tmp++;
      }
   }
   /* common case, no spaces or tabs */
   else
   {
      /* name starts from first char */
      name = tmp;
      while (*tmp != ' ' && *tmp != '\t' && *tmp != '\0')
      {
         tmp++;
      }
   }

   if (*tmp == '\0')                        /* no path */
   {
      sjog_cmd->name = strdup(name);
      if (sjog_cmd->name == NULL)
      {
         *err = TRUE;
         return NULL;
      }
      sjog_cmd->path = NULL;
      return sjog_cmd;
   }

   *tmp = '\0';

   /* copy the name */

   sjog_cmd->name = strdup(name);
   if (sjog_cmd->name == NULL)
   {
      *err = TRUE;
      return NULL;
   }

   /* strip the blanks */

   tmp++;
   while (*tmp == ' ' || *tmp == '\t' || *tmp == '"')
   {
      tmp++;
   }
   if (*tmp == '\0')                        /* premature end of line */
   {
      sjog_cmd->path = NULL;
      return sjog_cmd;
   }

   /* copy the path */

   sjog_cmd->path = strdup(tmp);
   if (sjog_cmd->path == NULL)
   {
      *err = TRUE;
      return NULL;
   }

   return sjog_cmd;
}

sjog_cmd_t *
sjog_cmd_new()
{
   sjog_cmd_t *sjog_cmd;

   sjog_cmd = (sjog_cmd_t *) malloc (sizeof(sjog_cmd_t));
   if (!sjog_cmd)
   {
      fprintf(stderr, "memory exhausted.\n");
      return NULL;
   }

   return sjog_cmd;
}

sjog_button_cmd_t *
sjog_button_cmd_new()
{
   sjog_button_cmd_t *sjog_button_cmd;

   sjog_button_cmd = (sjog_button_cmd_t *) malloc (sizeof(sjog_button_cmd_t));
   if (!sjog_button_cmd)
   {
      fprintf(stderr, "memory exhausted.\n");
      return NULL;
   }

   return sjog_button_cmd;
}

/*
 * modelines
 * ex:set ai ts=3 sw=3:
 * ei:set itab=spaces noat:
 * vim:set et:
 */
