/* probably redondant includes... */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xos.h>
#include <X11/keysym.h>
#include <X11/Xresource.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>
#include <dirent.h>
#include <stdarg.h>
#include <signal.h>
#include <sys/wait.h>

#include "sjog_scroll.h"

#include <libintl.h>
#define _(String) gettext (String)

Display *disp = NULL;        /* X11 display sjog is running on */
KeyCode upKeycode;           /* what to send when jog is up */
KeyCode downKeycode;         /* what to send when jog is down */
KeyCode backKeycode[2];         /* what to send when back button is pressed
                                   (two keys) */

/*
 * retrieve KeySym for page up/down then KeyCode.
 */
void
sjog_scroll_init_keycodes()
{

   KeySym ks;

   ks = XStringToKeysym("Page_Up");
   upKeycode = XKeysymToKeycode(disp, ks);

   ks = XStringToKeysym("Page_Down");
   downKeycode = XKeysymToKeycode(disp, ks);

   ks = XStringToKeysym("Alt_L");
   backKeycode[0] = XKeysymToKeycode(disp, ks);
   ks = XStringToKeysym("Left");
   backKeycode[1] = XKeysymToKeycode(disp, ks);
}

/* 
 * Sends Page_Up keyboard event through XTest extension. This won't
 * work if XTest is not available.
 */
void
sjog_scroll_top()
{

   D_ENTER;

   XTestGrabControl(disp, True);

   if (mousewheelFakeKeys)
   {
      XTestFakeKeyEvent(disp, upKeycode, True, 0);
   }
   else
   {
      XTestFakeButtonEvent(disp, 4, True, CurrentTime);
      XTestFakeButtonEvent(disp, 4, False, 30);
   }

   XSync(disp, False);
   XTestGrabControl(disp, False);

}

/* Sends Page_Down keyboard event through XTest extension. This won't
 * work if XTest is not available.
 */
void
sjog_scroll_bottom()
{

   D_ENTER;

   XTestGrabControl(disp, True);

   if (mousewheelFakeKeys)
   {
      XTestFakeKeyEvent(disp, downKeycode, True, 0);
   }
   else
   {
      XTestFakeButtonEvent(disp, 5, True, CurrentTime);
      XTestFakeButtonEvent(disp, 5, False, 30);
   }

   XSync(disp, False);
   XTestGrabControl(disp, False);

}

/* 
 * Sends Alt-Left when the back button is pressed.
 */ 
void
sjog_scroll_back()
{

   D_ENTER;

   XTestGrabControl(disp, True);
  
   /* Alt key down, left key down, left key up, alt key up */
   XTestFakeKeyEvent(disp, backKeycode[0], True, 0);
   XTestFakeKeyEvent(disp, backKeycode[1], True, 0);
   XTestFakeKeyEvent(disp, backKeycode[1], False, 30);
   XTestFakeKeyEvent(disp, backKeycode[0], False, 30);
   
   XSync(disp, False);
   XTestGrabControl(disp, False);
   
}

/*
 * Switch to list mode when jog is pushed.
 */
sjog_mode_t *
sjog_scroll_pushed()
{
   D_ENTER;

   return listMode;
}

/*
 * Initializes the display and the keycodes for up/down action and register
 * the callbacks for this mode
 */
void
sjog_scroll_mode_init(sjog_mode_t *scroll_mode)
{
   KeySym ks_up, ks_down;

   D_ENTER;

   if(disp == NULL)
   {
      /* try to open x11 display */
      disp = XOpenDisplay(NULL);
      if (!disp)
      {
         fprintf(stderr, _("Can't open X display. It *is* running, yeah?"));
         exit(1);
      }

      sjog_scroll_init_keycodes();
   }

   scroll_mode->bottom_cb = sjog_scroll_bottom;
   scroll_mode->pushed_cb = sjog_scroll_pushed;
   scroll_mode->top_cb = sjog_scroll_top;
   scroll_mode->back_cb = sjog_scroll_back;
   scroll_mode->start = NULL;
   scroll_mode->stop = NULL;
}

/*
 * modelines
 * ex:set ai ts=3 sw=3:
 * ei:set itab=spaces noat:
 * vim:set et:
 */
