#include "sjog_sonypi.h"

#include <libintl.h>
#define _(String) gettext (String)

/*
 * Open the device or exit if failed.
 */
void
sjog_sonypi_init()
{
   gboolean success = FALSE;
   char discard[BUFSIZ];

   /* 
      TT - First, we check to see if the sonypiDevice variable is not NULL.
      If it's set, then we try to open that device. Otherwise, we try the
      standard /dev/sonypi device, and finally the devfs /dev/misc/sonypi
      device.
    */

   if (sonypiDevice != NULL)
   {
      if ((sonypi_fd = open(sonypiDevice, O_RDONLY)) == -1)
      {
         fprintf(stderr, _("Failed to open %s: %s\n"), sonypiDevice,
                 strerror(errno));
         fprintf(stderr, _("Giving up...\n"));
         exit(1);
      }
      else
         success = TRUE;
   }

   if ((success != TRUE)
       && ((sonypi_fd = open("/dev/sonypi", O_RDONLY)) == -1))
   {
      fprintf(stderr, _("Failed to open /dev/sonypi: %s\n"), strerror(errno));
      fprintf(stderr, _("Trying /dev/misc/sonypi... "));
      if ((sonypi_fd = open("/dev/misc/sonypi", O_RDONLY)) == -1)
      {
         fprintf(stderr, _("\nFailed to open /dev/misc/sonypi: %s\n"),
                 strerror(errno));
         fprintf(stderr, _("Giving up...\n"));
         exit(1);
      }
      else
      {
         success = TRUE;
         fprintf(stderr, _("success\n"));
      }
   }
   else
      success = TRUE;

   /* Read and discard any data that is already on the device
    * This stops programs launching because the wheel has been
    * turned before sjog starts or whatever */
   fcntl(sonypi_fd, F_SETFL, O_NONBLOCK, 1);
   while(read(sonypi_fd, discard, sizeof(discard)) > 0);
   fcntl(sonypi_fd, F_SETFL, O_NONBLOCK, 0);
   
   gdk_input_add(sonypi_fd, GDK_INPUT_READ, sjog_sonypi_read, NULL);
}

/*
 * Reads the jog position by doing a select() on the device.
 */
void
sjog_sonypi_read(gpointer * data)
{
   unsigned char event;
   int len, i;
   sjog_button_cmd_t *cmd;

   D_ENTER;

   if (read(sonypi_fd, &event, sizeof(event)) != sizeof(event))
      fprintf(stderr, _("read failed: %s\n"), strerror(errno));

   switch (event)
   {
      case SONYPI_EVENT_JOGDIAL_DOWN:
         /* TT - Added the reverse spin capability */
         if (reverseSpinDirection)
         sjog_jog_position_changed(JOG_TOP);
         else
         sjog_jog_position_changed(JOG_BOTTOM);
         break;

      case SONYPI_EVENT_JOGDIAL_UP:
         /* TT - Added the reverse spin capability */
         if (reverseSpinDirection)
         sjog_jog_position_changed(JOG_BOTTOM);
         else
         sjog_jog_position_changed(JOG_TOP);
         break;

      case SONYPI_EVENT_JOGDIAL_DOWN_PRESSED:
         sjog_jog_position_changed(JOG_PUSHED);
         break;

      case SONYPI_EVENT_JOGDIAL_UP_PRESSED:
         sjog_jog_position_changed(JOG_PUSHED);
         break;

      case SONYPI_EVENT_JOGDIAL_PRESSED:
         sjog_jog_position_changed(JOG_PUSHED);
         break;

      case SONYPI_EVENT_JOGDIAL_BACK_PRESSED:
         sjog_jog_position_changed(JOG_BACK);
         break;

      default:
         /* fn keys and buttons */
         len = g_list_length(buttons);
         for (i=0; i<len; i++)
         {
            cmd = g_list_nth_data(buttons, i);
            if (cmd!=NULL && cmd->command==event)
            {
               sjog_mode_t * mode = sjog_exec_cmd(cmd->path);
               if(mode != NULL)
                  sjog_set_mode(mode);
            }
         }
   }
}

/*
* modelines
* ex:set ai ts=3 sw=3:
* ei:set itab=spaces noat:
* vim:set et:
*/
